<?php

// Exit if accessed directly
defined( 'ABSPATH' ) || exit;

require_once WCSS_PLUGIN_PATH . 'includes/legacy/abstract-wcss-legacy-add-to-cart.php';

class WCSS_Add_To_Cart extends WCSS_Legacy_Add_To_Cart {

	public function __construct() {
		if ( ! isset( $_GET['wcss_retrieve_cart'] ) ) {
			// Add rewrite rules
			add_action( 'init', array( $this, 'add_plugin_rewrite_rule' ) );

			// Add query variable
			add_filter( 'query_vars', array( $this, 'add_query_var' ) );
		}

		// Add products and coupons into cart
		add_action( 'wp', array( $this, 'add_products_to_cart' ) );

		add_action( 'wcss_after_add_products_to_cart', array( $this, 'delete_cart_after_retrieved' ), 30, 1 );

		add_action( 'woocommerce_after_cart_item_quantity_update', array( $this, 'remove_retrieved_cart_key_from_session' ) );
		add_action( 'woocommerce_remove_cart_item', array( $this, 'remove_retrieved_cart_key_from_session' ) );
		add_action( 'woocommerce_add_to_cart', array( $this, 'remove_retrieved_cart_key_from_session' ) );

		add_action( 'woocommerce_checkout_order_processed', array( $this, 'processed_retrieved_cart' ) );
	}

	/**
	 * Add query variable
	 *
	 * @since 1.0
	 */
	public function add_query_var( $vars ) {

		$vars[] = 'wcss_retrieve_cart';
		return $vars;
	}

	/**
	 * Add rewrite rules when WordPress is on init stage
	 *
	 * @since 1.0
	 */
	public function add_plugin_rewrite_rule() {

		// Get base slug
		$base_slug = get_option( 'wcss_base_slug' );

		// Add rewrite rule for plugin
		add_rewrite_rule( '^(' . $base_slug . ')/([^/]*)/?', 'index.php?wcss_retrieve_cart=$matches[2]', 'top' );
	}

	/**
	 * Add products and coupons into the cart when user click on cart shared link.
	 *
	 * @since 1.0
	 */
	public function add_products_to_cart() {
		if ( isset( $_GET['wcss_retrieve_cart'] ) ) {
			$cart_key = sanitize_text_field( $_GET['wcss_retrieve_cart'] );
		} else {
			$cart_key = get_query_var( 'wcss_retrieve_cart' );
		}

		if ( ! $cart_key ) {
			return;
		}

		do_action( 'wcss_before_add_products_to_cart' );

		// Empty user's current cart
		WC()->cart->empty_cart( true );

		// Get shared cart data from database
		$shared_cart = wcss_get_shared_cart( $cart_key );

		if ( ! $shared_cart->is_exists() || ! $shared_cart->get_cart_value() ) {
			wc_add_notice( esc_textarea( get_option( 'wcss_cart_not_found_message' ) ), 'error' );
			wp_redirect( wcss_add_to_cart_redirection_url() );
			exit;
		}

		// Update cart_retrieved
		wcss_shared_cart_retrieved_count_increment( $cart_key );

		// Update cart_retrieved_time
		wcss_set_shared_cart_retrieved_time( $cart_key );

		$cart_session = $shared_cart->get_cart_value();

		// Set the session
		WC()->session->cart            = $cart_session['cart'];
		WC()->session->cart_totals     = $cart_session['cart_totals'];
		WC()->session->applied_coupons = $cart_session['applied_coupons'];

		WC()->session->set( 'wcss_retrieved_cart_key', $cart_key );

		WC()->cart->get_cart_from_session();
		WC()->cart->calculate_totals();

		// Display cart retrieved message.
		wc_add_notice( esc_textarea( get_option( 'wcss_cart_retrieved_message' ) ), 'success' );

		/**
		 * Hook: wcss_after_add_products_to_cart.
		 *
		 * @hooked: WCSS_Add_To_Cart::delete_cart_after_retrieved() - 30
		 */
		do_action( 'wcss_after_add_products_to_cart', $cart_key );

		// redirect user on cart page when all products and coupons added into cart
		wp_redirect( wcss_add_to_cart_redirection_url() );

		exit;
	}

	/**
	 * Delete the cart after added to cart.
	 *
	 * @param string $cart_key
	 * @since 1.7.2
	 * @return void
	 */
	public function delete_cart_after_retrieved( $cart_key ) {
		$shared_cart = wcss_get_shared_cart( $cart_key );
		$saved_cart  = wcss_get_saved_cart( $cart_key );
		$options     = get_option( 'wcss_automatically_delete_cart_options' );

		// Delete saved and shared cart.
		if ( 'yes' === $options['shared_cart_after_retrieved'] && ! $saved_cart->is_exists() ) {
			$shared_cart->delete();
		}

		// Delete saved cart.
		if ( 'yes' === $options['saved_cart_after_retrieved'] ) {
			$shared_cart->delete();
			$saved_cart->delete();
		}

		// Delete only shared cart.
		if ( 'yes' === $options['shared_and_saved_cart_after_retrieved'] ) {
			$shared_cart->delete();
			$saved_cart->delete();
		}
	}

	/**
	 * Remove retrieved cart key from session.
	 *
	 * @since 2.1.0
	 *
	 * @return void
	 */
	public function remove_retrieved_cart_key_from_session() {
		if ( ! isset( WC()->session ) ) {
			return;
		}

		if ( ! WC()->session->__isset( 'wcss_retrieved_cart_key' ) ) {
			return;
		}

		WC()->session->__unset( 'wcss_retrieved_cart_key' );
	}

	/**
	 * Processed retrieved cart process.
	 *
	 * @since 2.1.0
	 *
	 * @return void
	 */
	public function processed_retrieved_cart( $order_id ) {
		if ( is_admin() ) {
			return;
		}

		if ( ! isset( WC()->session ) ) {
			return;
		}

		$retrieved_cart_key = WC()->session->get( 'wcss_retrieved_cart_key' );

		if ( ! $retrieved_cart_key ) {
			return;
		}

		wcss_add_shared_cart_meta( $retrieved_cart_key, 'is_cart_processed', 'yes' );
		wcss_add_shared_cart_meta( $retrieved_cart_key, 'cart_processed_by', get_current_user_id() );
		wcss_add_shared_cart_meta( $retrieved_cart_key, 'cart_processed_at', current_time( 'mysql' ) );
		wcss_add_shared_cart_meta( $retrieved_cart_key, 'cart_processed_order_id', absint( $order_id ) );

		/**
		 * Hook: Run when customer processed a retrieved cart.
		 *
		 * @since 2.1.0
		 *
		 * @param string $retrieved_cart_key Retrieved cart key.
		 */
		do_action( 'wcss_processed_retrieved_cart', $retrieved_cart_key );
	}
}

$wcss_add_to_cart = new WCSS_Add_To_Cart();
