<?php

// Exit if accessed directly
defined( 'ABSPATH' ) || exit;

class WCSS_Email_Cart {

	public function __construct() {
		// Email Header, Footer and content hooks.
		add_action( 'wcss_email_cart_header', array( $this, 'email_header' ) );
		add_action( 'wcss_email_cart_details', array( $this, 'email_cart_retrieve' ), 10 );
		add_action( 'wcss_email_cart_details', array( $this, 'email_cart_products' ), 20 );
		add_action( 'wcss_email_cart_details', array( $this, 'email_cart_total' ), 30 );
		add_action( 'wcss_email_cart_footer', array( $this, 'email_footer' ) );

		// Hooks for sending emails via AJAX.
		add_action( 'wp_ajax_woocommerce_wcss_send_email_cart', array( $this, 'email_cart' ) );
		add_action( 'wp_ajax_woocommerce_nopriv_wcss_send_email_cart', array( $this, 'email_cart' ) );
		add_action( 'wc_ajax_wcss_send_email_cart', array( $this, 'email_cart' ) );

		// Let 3rd parties unhook the above via this hook.
		do_action( 'wcss_email_cart', $this  );
	}

	/**
	 * Email cart.
	 */
	public function email_cart() {
		if ( ! isset( $_POST['wcss_email_cart_nonce'] ) || ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['wcss_email_cart_nonce'] ) ), 'wcss_email_cart' ) ) {
			wp_send_json_error( __( 'Invalid nonce', 'woo-cart-share' ) );
		}

		$response = array();
		$cart_key = sanitize_text_field( wp_unslash( $_POST['cart_key'] ) );

		$post = array(
			'email_to'      => sanitize_text_field( $_POST['email_to'] ),
			'email_subject' => sanitize_text_field( $_POST['email_subject'] ),
			'email_message' => wp_kses_post( $_POST['email_message'] ),
		);

		/**
		 * Filter posted data.
		 */
		$post = apply_filters( 'wcss_email_cart_form_post_data', $post );

		$email_from_name  = apply_filters( 'wcss_email_from_name', get_option( 'wcss_email_from_name' ) );
		$email_from_email = apply_filters( 'wcss_email_from_email', get_option( 'wcss_email_from_email' ) );

		wcss_set_session_to_cart( $cart_key );

		$email_body = wcss_get_template_html(
			'email-cart/email-cart.php',
			array(
				'cart_key'                  => $cart_key,
				'message'                   => isset( $post['email_message'] ) ? $post['email_message'] : '',

				// Following variables were removed in plugin version 2.3.0
				'brand_name'                => get_option( 'wcss_email_brand_name' ),
				'brand_logo_url'            => get_option( 'wcss_email_brand_logo_url' ),
				'retrieve_cart_button_text' => get_option( 'wcss_email_retrieve_cart_button_text' ),
			)
		);

		// Email headers.
		$headers[] = 'Content-Type: text/html; charset=UTF-8';
		$headers[] = 'From: ' . esc_html( $email_from_name ) . ' <' . esc_html( $email_from_email ) . '>';

		/**
		 * Filter email cart headers.
		 *
		 * @since 1.9.1
		 *
		 * @param array $headers Email cart header.
		 * @return array Email cart header.
		 */
		$headers = apply_filters( 'wcss_email_cart_headers', $headers );

		/**
		 * Before email cart send action.
		 */
		do_action( 'wcss_before_send_email_cart', $_POST['cart_key'], $_POST, $headers );

		$email_to = explode( ',', $post['email_to'] );

		// Send multiple emails.
		foreach ( $email_to as $email ) {
			$email = trim( $email );

			if ( ! $email ) {
				continue;
			}
			if ( ! is_email( $email ) ) {
				continue;
			}

			wp_mail( $email, $post['email_subject'], wcss_email_style_inline( $email_body ), $headers );
		}

		$response['status'] = true;

		/**
		 * After email cart send action.
		 */
		do_action( 'wcss_after_send_email_cart', $_POST['cart_key'], $_POST, $headers );

		echo json_encode( $response );

		wp_die();
	}

	/**
	 * Email cart header template.
	 *
	 * @since 2.3.0
	 *
	 * @return void
	 */
	public function email_header( $message ) {
		wcss_get_template( 'email-cart/header.php', array( 'message' => $message ) );
	}

	/**
	 * Email cart footer template.
	 *
	 * @since 2.3.0
	 *
	 * @return void
	 */
	public function email_footer() {
		wcss_get_template( 'email-cart/footer.php' );
	}

	/**
	 * Email cart retrieve template.
	 *
	 * @since 2.3.0
	 *
	 * @return void
	 */
	public function email_cart_retrieve( $cart_key ) {
		wcss_get_template( 'email-cart/retrieve-cart.php', array( 'cart_key' => $cart_key ) );
	}

	/**
	 * Email cart products template.
	 *
	 * @since 2.3.0
	 *
	 * @return void
	 */
	public function email_cart_products() {
		wcss_get_template( 'email-cart/cart-products.php' );
	}

	/**
	 * Email cart total template.
	 *
	 * @since 2.3.0
	 *
	 * @return void
	 */
	public function email_cart_total() {
		wcss_get_template( 'email-cart/cart-totals.php' );
	}
}

$wcss_email_cart = new WCSS_Email_Cart();
