<?php

// Exit if accessed directly
defined( 'ABSPATH' ) || exit;

require_once WCSS_PLUGIN_PATH . 'includes/legacy/class-wcss-legacy-saved-cart.php';

/**
 * Class WCSS_Saved_Cart.
 */
class WCSS_Saved_Cart extends WCSS_Legacy_Saved_Cart {

	/**
	 * Contains the saved cart table name.
	 *
	 * @since 2.0.0
	 *
	 * @var string Saved cart table name.
	 */
	protected $table = 'wcss_saved_cart';

	/**
	 * Contains the saved cart data.
	 *
	 * @since 2.0.0
	 *
	 * @var array Saved cart data.
	 */
	protected $data = array(
		'id'         => 0,
		'user_id'    => 0,
		'cart_name'  => '',
		'cart_key'   => '',
		'cart_saved' => '',
	);

	/**
	 * Class constructor.
	 *
	 * @since 2.0.0
	 *
	 * @param string $cart_key Optional. Saved cart key.
	 */
	public function __construct( $cart_key = '' ) {
		parent::__construct();

		if ( $cart_key ) {
			$this->read( $cart_key );
		}
	}

	/**
	 * --------------------------------
	 *  GETTERS METHODS
	 * --------------------------------
	 */

	/**
	 * Get saved cart ID.
	 *
	 * @return int Saved cart ID.
	 */
	public function get_id() {
		return absint( $this->data['id'] );
	}

	/**
	 * Get saved cart user ID.
	 *
	 * @return int Saved cart user ID.
	 */
	public function get_user_id() {
		return absint( $this->data['user_id'] );
	}

	/**
	 * Get saved cart name.
	 *
	 * @return string Saved cart name.
	 */
	public function get_cart_name() {
		return $this->data['cart_name'];
	}

	/**
	 * Get saved cart key.
	 *
	 * @return string Saved cart key.
	 */
	public function get_cart_key() {
		return $this->data['cart_key'];
	}

	/**
	 * Get saved cart time.
	 *
	 * @return string Saved cart time.
	 */
	public function get_cart_saved() {
		return $this->data['cart_saved'];
	}

	/**
	 * --------------------------------
	 *  SETTERS METHODS
	 * --------------------------------
	 */

	/**
	 * Set saved cart ID.
	 *
	 * @since 2.0.0
	 *
	 * @param int $id Saved cart ID.
	 * @return void
	 */
	public function set_id( $id ) {
		$this->data['id'] = absint( $id );
	}

	/**
	 * Set saved cart user ID.
	 *
	 * @since 2.0.0
	 *
	 * @param int $user_id Saved cart user ID.
	 * @return void
	 */
	public function set_user_id( $user_id ) {
		$this->data['user_id'] = absint( $user_id );
	}

	/**
	 * Set saved cart name.
	 *
	 * @since 2.0.0
	 *
	 * @param string $cart_name Saved cart name.
	 * @return void
	 */
	public function set_cart_name( $cart_name ) {
		$this->data['cart_name'] = $cart_name;
	}

	/**
	 * Set saved cart key.
	 *
	 * @since 2.0.0
	 *
	 * @param string $cart_key Saved cart key.
	 * @return void
	 */
	public function set_cart_key( $cart_key ) {
		$this->data['cart_key'] = $cart_key;
	}

	/**
	 * Set saved cart time.
	 *
	 * @since 2.0.0
	 *
	 * @param string $cart_saved Saved cart time.
	 * @return void
	 */
	public function set_cart_saved( $cart_saved ) {
		$this->data['cart_saved'] = $cart_saved;
	}

	/**
	 * --------------------------------
	 *  CONDITIONAL METHODS
	 * --------------------------------
	 */

	/**
	 * Check whether the saved cart is exists or not.
	 *
	 * @since 2.0.0
	 *
	 * @return bool True if saved cart exists, false otherwise.
	 */
	public function is_exists() {
		return ( $this->get_id() > 0 );
	}

	/**
	 * --------------------------------
	 *  OTHER METHODS
	 * --------------------------------
	 */

	/**
	 * --------------------------------
	 *  CURD METHODS
	 * --------------------------------
	 */

	/**
	 * Create a new saved cart.
	 *
	 * @since 2.0.0
	 *
	 * @return WCSS_Saved_Cart|bool Current instance of WCSS_Saved_Cart, false otherwise.
	 */
	public function create() {
		global $wpdb;

		if ( ! $this->get_user_id() ) {
			$this->set_user_id( get_current_user_id() );
		}
		if ( ! $this->get_cart_saved() ) {
			$this->set_cart_saved( current_time( 'mysql' ) );
		}

		$create = $wpdb->insert(
			$wpdb->prefix . $this->table,
			array(
				'user_id'    => $this->get_user_id(),
				'cart_name'  => $this->get_cart_name(),
				'cart_key'   => $this->get_cart_key(),
				'cart_saved' => $this->get_cart_saved(),
			)
		);

		if ( ! $create ) {
			return false;
		}

		$this->set_id( $wpdb->insert_id );

		return $this;
	}

	/**
	 * Read a saved cart from database.
	 *
	 * @since 2.0.0
	 *
	 * @param string $cart_key Saved cart key.
	 * @return void
	 */
	public function read( $cart_key ) {
		global $wpdb;

		$read = $wpdb->get_row(
			$wpdb->prepare(
				"SELECT * FROM {$wpdb->prefix}{$this->table} WHERE cart_key = %s",
				$cart_key
			)
		);

		if ( ! $read ) {
			return;
		}

		$this->set_id( $read->id );
		$this->set_user_id( $read->user_id );
		$this->set_cart_name( $read->cart_name );
		$this->set_cart_key( $read->cart_key );
		$this->set_cart_saved( $read->cart_saved );
	}

	/**
	 * Update a saved cart.
	 *
	 * @since 2.0.0
	 *
	 * @return WCSS_Saved_Cart|bool Current instance of WCSS_Saved_Cart, false otherwise.
	 */
	public function update() {
		global $wpdb;

		$update = $wpdb->update(
			$wpdb->prefix . $this->table,
			array(
				'user_id'    => $this->get_user_id(),
				'cart_name'  => $this->get_cart_name(),
				'cart_key'   => $this->get_cart_key(),
				'cart_saved' => $this->get_cart_saved(),
			),
			array(
				'id' => $this->get_id(),
			)
		);

		if ( ! $update ) {
			return false;
		}

		return $this;
	}

	/**
	 * Delete a saved cart.
	 *
	 * @since 2.0.0
	 *
	 * @return WCSS_Saved_Cart|bool Current instance of WCSS_Saved_Cart, false otherwise.
	 */
	public function delete() {
		global $wpdb;

		$delete = $wpdb->delete(
			$wpdb->prefix . $this->table,
			array(
				'id' => $this->get_id(),
			)
		);

		if ( ! $delete ) {
			return false;
		}

		return $this;
	}

}
