<?php

// Exit if accessed directly
defined( 'ABSPATH' ) || exit;

if ( ! function_exists( 'wcss_current_time' ) ) {
	/**
	 * Get current time and date.
	 *
	 * Default: Month date, Year H:M am|pm
	 *
	 * @since  1.7
	 * @return string return current time with format.
	 */
	function wcss_current_time() {
		return current_time( apply_filters( 'wcss_date_format', 'M j, Y g:i a' ) );
	}
}

if ( ! function_exists( 'wcss_get_cart_link' ) ) {
	/**
	 * Get complete share cart link by cart key.
	 *
	 * @param string $cart_key
	 * @return string
	 */
	function wcss_get_cart_link( $cart_key ) {
		$cart_share_link = home_url() . '/' . get_option( 'wcss_base_slug' ) . '/' . $cart_key;
		return esc_url( $cart_share_link );
	}
}

if ( ! function_exists( 'wcss_get_template' ) ) {
	/**
	 * Get plugin templates.
	 *
	 * @param string $template_name relative path
	 * @param array  $args
	 */
	function wcss_get_template( $template_name, $args = array() ) {
		extract( $args );

		$template_file = get_stylesheet_directory() . '/wcss-templates/' . $template_name;

		if ( file_exists( $template_file ) ) {
			$template = $template_file;
		} else {
			$template = trailingslashit( WCSS_PLUGIN_PATH ) . 'templates/' . $template_name;
		}

		require apply_filters( 'wcss_get_template', $template, $template_name, $args );
	}
}

if ( ! function_exists( 'wcss_get_template_html' ) ) {
	/**
	 * Get plugin templates.
	 *
	 * @param string $template_name relative path
	 * @param array  $args
	 */
	function wcss_get_template_html( $template_name, $args = array() ) {
		ob_start();
		wcss_get_template( $template_name, $args );
		return apply_filters( 'wcss_get_template_html', ob_get_clean(), $template_name, $args );
	}
}

if ( ! function_exists( 'wcss_apply_inline_styles' ) ) {
	/**
	 * Apply inline styles to dynamic html.
	 *
	 * We only inline CSS for html emails, and to do so we use Emogrifier library (if supported).
	 *
	 * @param string|null $html html that will receive inline styles.
	 * @return string
	 */
	function wcss_apply_inline_styles( $html = '', $css = '' ) {

		if ( class_exists( 'DOMDocument' ) && version_compare( PHP_VERSION, '5.5', '>=' ) ) {

			if ( ! class_exists( 'Pelago\\Emogrifier' ) ) {
				require_once WCSS_PLUGIN_PATH . 'includes/libraries/emogrifier/class-wcss-emogrifier.php';
			}
			// Apply Emogrifier to inline the CSS.
			$emogrifier = new Pelago\Emogrifier();
			$emogrifier->setHtml( $html );
			$emogrifier->setCss( strip_tags( $css ) );
			$html = $emogrifier->emogrify();
		} else {
			$html = '<style type="text/css">' . $css . '</style>' . $html;
		}

		return $html;
	}
}
if ( ! function_exists( 'wcss_get_current_user_role' ) ) {
	/**
	 * Get the current user role.
	 *
	 * @return bool|int
	 */
	function wcss_get_current_user_role() {
		if ( is_user_logged_in() ) {
			$user = wp_get_current_user();
			$role = (array) $user->roles;
			return $role[0];
		} else {
			return false;
		}
	}
}

if ( ! function_exists( 'wcss_currnet_role_has_access' ) ) {
	/**
	 * Check whether current user has ability to share the cart or not.
	 *
	 * @return bool
	 */
	function wcss_currnet_role_has_access() {
		if ( get_option( 'wcss_display_by_role_status' ) === 'yes' ) {
			if ( ! in_array( wcss_get_current_user_role(), get_option( 'wcss_display_by_role' ) ) ) {
				return false;
			}
		}

		return true;
	}
}

if ( ! function_exists( 'wcss_share_integration_option' ) ) {
	/**
	 * Get share media integration options.
	 *
	 * @since 1.8.3
	 *
	 * @param string $share_medium
	 * @param string $data
	 * @return void
	 */
	function wcss_share_integration_option( $share_medium, $data = '' ) {
		if ( ! $option = get_option( 'wcss_share_integration' ) ) {
			return false;
		}

		if ( ! $share_medium ) {
			return false;
		}

		if ( isset( $option[ $share_medium ][ $data ] ) ) {
			return $option[ $share_medium ][ $data ];
		}

		if ( isset( $option[ $share_medium ] ) ) {
			return $option[ $share_medium ];
		}

		return $option;
	}
}

if ( ! function_exists( 'wcss_set_session_to_cart' ) ) {
	/**
	 * Set saved session to cart.
	 *
	 * @since 1.9.0
	 *
	 * @param string $cart_key Shared cart key.
	 * @return void
	 */
	function wcss_set_session_to_cart( $cart_key ) {
		// Empty user's current cart
		WC()->cart->empty_cart( true );

		// Get shared cart data from database
		$cart_session = wcss_get_shared_cart_data( $cart_key, 'cart_value' );

		if ( $cart_session ) {
			// Set the session
			WC()->session->cart            = $cart_session['cart'];
			WC()->session->cart_totals     = $cart_session['cart_totals'];
			WC()->session->applied_coupons = $cart_session['applied_coupons'];

			WC()->cart->get_cart_from_session();
			WC()->cart->calculate_totals();
		}

	}
}

if ( ! function_exists( 'wcss_get_current_cart_session' ) ) {

	/**
	 * Get current cart session.
	 *
	 * @since 1.9.1
	 *
	 * @see WC_Session_Handler::get_cart_for_session();
	 *
	 * @return array Current cart session.
	 */
	function wcss_get_current_cart_session() {
		return WC()->session->get_session_data();
	}
}

if ( ! function_exists( 'wcss_set_current_cart_session' ) ) {
	/**
	 * Set current cart session.
	 *
	 * @since 1.9.1
	 *
	 * @return void
	 */
	function wcss_set_current_cart_session( $current_cart_session ) {
		// Empty user's current cart
		WC()->cart->empty_cart( true );

		if ( isset( $current_cart_session['cart'] ) ) {
			WC()->session->cart = maybe_unserialize( $current_cart_session['cart'] );
		}
		if ( isset( $current_cart_session['cart_totals'] ) ) {
			WC()->session->cart_totals = maybe_unserialize( $current_cart_session['cart_totals'] );
		}
		if ( isset( $current_cart_session['applied_coupons'] ) ) {
			WC()->session->applied_coupons = maybe_unserialize( $current_cart_session['applied_coupons'] );
		}

		WC()->cart->get_cart_from_session();
		WC()->cart->calculate_totals();
	}
}

if ( ! function_exists( 'wcss_get_saved_cart_endpoint_url' ) ) {
	/**
	 * Get saved cart endpoint URL.
	 *
	 * @since 1.9.3
	 *
	 * @return string Saved cart endpoint URL.
	 */
	function wcss_get_saved_cart_endpoint_url() {
		return apply_filters( 'wcss_saved_cart_endpoint_url', wc_get_account_endpoint_url( get_option( 'wcss_my_account_tab_slug' ) ) );
	}
}

/**
 * Get date in format.
 *
 * @since 2.0.0
 *
 * @param string $date A valid date or timestamp.
 * @param string $format Optional. Date format. Default: 'M j, Y g:i a'.
 * @return string Date in format.
 */
function wcss_date( $date, $format = 'M j, Y g:i a' ) {
	if ( ! $date ) {
		return false;
	}

	if ( ! is_numeric( $date ) ) {
		$date = strtotime( $date );
	}

	return gmdate( apply_filters( 'wcss_date_format', $format ), $date );
}
